#!/usr/bin/perl

package UiO::NetApp;
use strict;
use warnings;
use Sys::Hostname;
use Exporter;
use Carp;
use version;
use Readonly;

# Functions provided:
#
#   my_netapps
#   my_tsm_netapps
#   is_netapp(host)
#   is_cifs_netapp(host)
#   on_netapp(path)
#   cifs_home_sessions(netapp, user)

use vars qw($VERSION @EXPORT %netapp_master %netapp_tsm @all_netapps @cifs_netapps);
use base qw(Exporter);

Readonly $VERSION => qv('v0.0.1');
Readonly @EXPORT
  => qw(my_netapps
	my_tsm_netapps
	is_netapp
	is_cifs_netapp
	on_netapp
	cifs_home_sessions
      );

# NetApp masters and their respective netapps
Readonly %netapp_master
  => (
      'schopenhauer' => [ 'aristoteles',
			  'arkimedes',
			  'platon',
			  'sokrates',
			  'utgard',
			  'midgard',
			  'asgard',
			  'bifrost',
			  'fronter-netapp01',
			  'fronter-netapp02' ],
      'ninhursaga'   => [ 'anatu', 'sarpanitu' ],
     );

# Backup nodes and their respective netapps
#
# The backup nodes for fronter-netapp01 and -02 aren't handled yet, some
# testing and modifications needs to be done in /local/sbin/check-tsm first.
# Ditto for anatu (ninhursaga).
#
# hh, 2010-11-25
Readonly %netapp_tsm
  => (
      'netapp-bu01' => [ 'aristoteles',
			 'arkimedes',
			 'platon',
			 'sokrates',
			 'asgard',
			 'bifrost',
			 'utgard',
			 'midgard' ],
     );

# All NetApps
Readonly @all_netapps
  => (
      'aristoteles',
      'arkimedes',
      'platon',
      'sokrates',
      'utgard',
      'midgard',
      'asgard',
      'bifrost',
      'fronter-netapp01',
      'fronter-netapp02',
      'anatu',
      'sarpanitu',
     );

# NetApps that exports CIFS shares
Readonly @cifs_netapps
  => (
      'aristoteles',
      'arkimedes',
      'platon',
      'sokrates',
      'asgard',
      'bifrost',
      'anatu',
      'sarpanitu',
     );

#=====================================================================
# Exported functions
#=====================================================================

# Returns an array of the NetApps that this host administers
sub my_netapps {
    my $hostname = hostname;
    $hostname =~ s{\.uio\.no \z}{}xms;

    return defined $netapp_master{$hostname}
      ? @{ $netapp_master{$hostname} }
	: ();
}

# Returns an array of the NetApps that this host is a TSM backup node for.
sub my_tsm_netapps {
    my $hostname = hostname;
    $hostname =~ s{\.uio\.no \z}{}xms;

    return defined $netapp_tsm{$hostname}
      ? @{ $netapp_tsm{$hostname} }
	: ();
}

# Returns true if a host is a known NetApp, otherwise false
sub is_netapp {
    my $host = shift;
    return if !defined $host;
    return scalar grep { $host eq $_ } @all_netapps;
}

# Returns true if a host is a NetApp providing CIFS service, otherwise false
sub is_cifs_netapp {
    my $host = shift;
    return if !defined $host;
    return scalar grep { $host eq $_ } @cifs_netapps;
}

# If the hostname part of a directory path is a NetApp, returns the hostname,
# otherwise 0 or undef.
sub on_netapp {
    my $path = shift;
    return if !defined $path;

    my $valid_path
      = qr{
	      \A
	      /[^/]+?    # fak
	      /([^/]+?)  # maskin
	      /[^/]+     # disk
	      \z
      }xms;

    my $host = undef;
    if ($path =~ m{$valid_path}xms) {
	$host = $1;
    }

    return is_netapp($host) ? $host : 0;
}

# Returns an array of the client addresses of a user's CIFS sessions
# to his/her home directory on a NetApp, if any.
sub cifs_home_sessions {
    my ($netapp, $user) = @_;

    return if (!defined$netapp || !defined $user);
    return if !is_cifs_netapp($netapp);
    return if $user !~ m{\A [a-z0-9_-]{2,8} \z}xms;

    # Checks the sessions by parsing the output from 'cifs sessions <user>'
    # run on the NetApp. Assuming a format from 'cifs sessions' like:
    #
    # | users
    # |   shares/files opened
    # |
    # | 129.240.203.70() (UIO\hh - hh)
    # |   {home_dir} /vol/usitusers/bsd-u1
    # |
    # |
    # | 129.240.5.37() (UIO\hh - hh)
    # |   {home_dir} /vol/usitusers/bsd-u1
    # |                  R&W-denyN   - \.notes\data\names.nsf
    # |                  R&W-denyN   - \.notes\data\headline.nsf

    my $cmd     = "rsh $netapp cifs sessions $user";
    my $client  = undef;
    my @clients = ();

    open my $SESSINFO, '-|', $cmd
      or do { carp "Can't do '$cmd': $!\n"; return };
    while (<$SESSINFO>) {
	chop;
	if (m{\A ((\d+\.){3}\d+)\(}xms) {
	    $client = $1;
	} elsif (m{\A \s+\{home_dir\}\s+/vol/}xms) {
	    if (defined($client)) {
		push @clients, $client;
	    }
	} elsif (m{\A \s* \z}xms) {
            # we're not in a block of session information any more
	    $client = undef;
	}
    }
    close $SESSINFO
      or do {
	  if ($!) { carp "Error closing pipe from '$cmd': $!\n"; }
	  else    { carp "Exit status $? from '$cmd'\n"; }
	  return
      };

    return \@clients;
}

1;
