#!/usr/bin/perl
#
# This modules defines functions that often are useful when doing
# various operations that involve home directories on the form
# /department/hostname/disk/username (which is currently the case for
# all regular users here on the University of Oslo, Norway).
#
# Created 2000-08-01 by Harald Meland <harald.meland@usit.uio.no>
#
# Sanitized 2014 by Trond Hasle Amundsen <t.h.amundsen@usit.uio.no>

package UiO::Pathname;
use strict;
use warnings;
use Exporter ();
use UiO::Host qw(contact_cmd);

use version;
our $VERSION = qv('0.0.2');

use vars qw(@ISA @EXPORT_OK @EXPORT);
@ISA = qw(Exporter);
@EXPORT_OK = qw(contact_host home_host split_path);
@EXPORT = @EXPORT_OK;


# Hash mapping a few special-case homedir hosts' FQDNs to FQDNs of the
# hosts that should be used to access those homedirs.
my %nas_master_host
  = (
     'hume.uio.no'             => 'hume-master.uio.no',
     'kant.uio.no'             => 'kant-master.uio.no',
     'anatu.uio.no'            => 'anatu-master.uio.no',
     'aristoteles.uio.no'      => 'aristoteles-master.uio.no',
     'arkimedes.uio.no'        => 'arkimedes-master.uio.no',
     'asgard.uio.no'           => 'asgard-master.uio.no',
     'asgard.ifi.uio.no'       => 'asgard-master.uio.no',
     'bifrost.uio.no'          => 'bifrost-master.uio.no',
     'bifrost.ifi.uio.no'      => 'bifrost-master.uio.no',
     'fronter-netapp01.uio.no' => 'fronter-netapp01-master.uio.no',
     'fronter-netapp02.uio.no' => 'fronter-netapp02-master.uio.no',
     'midgard.uio.no'          => 'midgard-master.uio.no',
     'platon.uio.no'           => 'platon-master.uio.no',
     'sokrates.uio.no'         => 'sokrates-master.uio.no',
     'sarpanitu.uio.no'        => 'sarpanitu-master.uio.no',
     'utgard.uio.no'           => 'utgard-master.uio.no',
    );

# Return a (hopefully) canonicalized version of a pathname string.
sub clean {
    my $dir = shift;
    return if ! defined $dir;

    # Collapse multiple slashes into a single one
    $dir =~ s{/+}{/}gxms;

    # Remove internal path components consisting of a single "."
    $dir =~ s{/\./}{/}gxms;

    # NOTE: As long as this function isn't doing any tests against a
    # file system, the possibility of a path component being a symlink
    # stops us from collapsing "/foo/../" into "/".

    # Collapse trailing "/." into "/"
    $dir =~ s{/\. \z}{/}xms;

    # Remove trailing "/" unless $dir eq "/"
    $dir =~ s{/ \z}{}xms if $dir ne '/';

    return $dir;
}


# Return the components of a pathname as a list.
#
# If the pathname is empty, return undef.
#
# Otherwise, some care is taken to make sure that the first element of
# the returned list is neither undef nor the empty string.
#
# This implies that it's up to the caller to keep track of whether
# it's dealing with absolute pathnames or not; the lists returned for
# "/foo/bar" and "foo/bar" are equal.
sub split_path {
    my $dir = shift;

    $dir = clean($dir);
    return if ! defined $dir;

    my @parts = split q{/}, $dir;

    # Iff $dir started with "/", the first element of @parts will be
    # the empty string.  We don't need that element.
    if (defined $parts[0] && $parts[0] eq q{}) {
	shift @parts;
    }

    return @parts;
}


# Return FQDN of host where the home directory or mount point $dir
# (which must follow the naming convention /department/host/disk) is
# local.
sub home_host {
    my $dir = shift;

    return if !defined $dir;
    return if $dir !~ m{\A /}xms;

    my ($dep, $host, $disk) = split_path($dir);

    # Fail if there weren't enough path components.
    return if !defined $disk;

    if ($dep eq 'ifi') {
	return "$host.ifi.uio.no";
    }

    return "$host.uio.no";
}


# When called in a scalar context, return FQDN of a Unix host where
# the home directory or mount point $dir should be local or
# root-mounted.
#
# Otherwise (i.e. when called in a list context), return a list of two
# values:
#   FQDN of the Unix host to contact and path to "remote shell"
#   command to use when contacting that Unix host.
sub contact_host {
    my $dir = shift;
    my $host = home_host($dir);

    return if !defined $host;

    # As always, there are some special cases that need extra
    # special care.
    if (defined $nas_master_host{$host}) {
	$host = $nas_master_host{$host};
    }

    # If $host is a CNAME, we return the name of the A record to ease 
    # comparisons.
    $host = (gethostbyname($host))[0];
    return wantarray ? ($host, contact_cmd($host)) : $host;
}

1;
